@@ -462,7 +462,7 @@ gps_handle_error_idle(gchar *error)
         {
             /* Disable GPS. */
             gtk_check_menu_item_set_active(
-                    GTK_CHECK_MENU_ITEM(_menu_enable_gps_item), FALSE);
+                    GTK_CHECK_MENU_ITEM(_menu_gps_enable_item), FALSE);
         }
 
         /* Ask user to re-connect. */
@@ -607,7 +607,9 @@ thread_read_nmea(GpsRcvrInfo *gri)
             gpsd_ctrl_sock = getenv("GPSD_CTRL_SOCK");
 
             if (!gpsd_prog)
-                gpsd_prog = "gpsd";
+                gpsd_prog = "/usr/sbin/gpsd";
+            if (!gpsd_ctrl_sock)
+                gpsd_ctrl_sock = "/tmp/.gpsd_ctrl_sock";
 
             memset(&gps_context, 0, sizeof(gps_context));
             errno = 0;
