@@ -58,8 +58,8 @@
 #include "poi.h"
 #include "util.h"
 
-typedef struct _OriginToggleInfo OriginToggleInfo;
-struct _OriginToggleInfo {
+typedef struct _RouteDownloadInfo RouteDownloadInfo;
+struct _RouteDownloadInfo {
     GtkWidget *rad_use_gps;
     GtkWidget *rad_use_route;
     GtkWidget *rad_use_text;
@@ -653,8 +653,10 @@ route_download_and_setup(GtkWidget *parent, const gchar *source_url,
     }
 
     /* Attempt to download the route from the server. */
-    if(GNOME_VFS_OK != (vfs_result = gnome_vfs_read_entire_file(
-                buffer, &size, &bytes)))
+    vfs_result = gnome_vfs_read_entire_file(buffer, &size, &bytes);
+    g_free (buffer);
+
+    if(vfs_result != GNOME_VFS_OK)
     {
         g_free(bytes);
         popup_error(parent, gnome_vfs_result_to_string(vfs_result));
@@ -751,49 +753,44 @@ track_add(time_t time, gboolean newly_fixed)
     printf("%s(%d, %d, %d, %d)\n", __PRETTY_FUNCTION__,
             (guint)time, newly_fixed, _pos.unitx, _pos.unity);
 
-    if(!time)
-    {
-        /* This is a null point. */
-        MACRO_PATH_INCREMENT_TAIL(_track);
-        *_track.tail = _point_null;
-    }
-    else
-    {
-        gboolean moving = FALSE;
-        gboolean approaching_waypoint = FALSE;
-        gint xdiff, ydiff, dopcand;
+    gboolean moving = FALSE;
+    gboolean approaching_waypoint = FALSE;
+    gint xdiff, ydiff, dopcand;
 
-        announce_thres_unsquared = (20+_gps.speed) * _announce_notice_ratio*32;
+    announce_thres_unsquared = (20+_gps.speed) * _announce_notice_ratio*32;
 
-        if(!_track.tail->unity
-                || ((xdiff = _pos.unitx - _track.tail->unitx), /* comma op */
-                    (ydiff = _pos.unity - _track.tail->unity), /* comma op */
-                    /* Check if xdiff or ydiff are huge. */
-                    ((abs(xdiff) >> 12) || (abs(ydiff) >> 12)
-                    /* Okay, let's see if we've moved enough to justify adding
-                     * to the track.  It depends on our error.  I'd like to
-                     * make the threshold roughly linear with respect to the
-                     * P/HDOP (completely arbitrary, I know), but I also
-                     * want to keep the threshold at a minimum of 2
-                     * zoom-level-4 pixel, and I want dop's of less than 2 to
-                     * also have a 1-pixel threshold.  I also throw in some
-                     * PDOP into the mix, just for fun. */
-                    || ((dopcand = 8 * (_gps.pdop - 6 +(_gps.hdop*_gps.hdop))),
-                        ((xdiff * xdiff) + (ydiff * ydiff)
-                             >= (MAX(2, dopcand) << 8))))))
+    if(!_track.tail->unity
+            || ((xdiff = _pos.unitx - _track.tail->unitx), /* comma op */
+                (ydiff = _pos.unity - _track.tail->unity), /* comma op */
+                /* Check if xdiff or ydiff are huge. */
+                ((abs(xdiff) >> 12) || (abs(ydiff) >> 12)
+                /* Okay, let's see if we've moved enough to justify adding
+                 * to the track.  It depends on our error.  I'd like to
+                 * make the threshold roughly linear with respect to the
+                 * P/HDOP (completely arbitrary, I know), but I also
+                 * want to keep the threshold at a minimum of 2
+                 * zoom-level-4 pixel, and I want dop's of less than 2 to
+                 * also have a 1-pixel threshold.  I also throw in some
+                 * PDOP into the mix, just for fun. */
+                || ((dopcand = 8 * (_gps.pdop - 6 +(_gps.hdop*_gps.hdop))),
+                    ((xdiff * xdiff) + (ydiff * ydiff)
+                         >= (MAX(2, dopcand) << 8))))))
+    {
+        /* We moved enough to actually register a move. */
+        ret = TRUE;
+
+        /* Update the nearest-waypoint data. */
+        if(_route.head != _route.tail
+                && (newly_fixed ? (route_find_nearest_point(), TRUE)
+                                : route_update_nears(TRUE)))
         {
-            /* We moved enough to actually register a move. */
-            ret = TRUE;
+            /* Nearest waypoint has changed - re-render paths. */
+            map_render_paths();
+            MACRO_QUEUE_DRAW_AREA();
+        }
 
-            /* Update the nearest-waypoint data. */
-            if(_route.head != _route.tail
-                    && (newly_fixed ? (route_find_nearest_point(), TRUE)
-                                    : route_update_nears(TRUE)))
-            {
-                /* Nearest waypoint has changed - re-render paths. */
-                map_render_paths();
-                MACRO_QUEUE_DRAW_AREA();
-            }
+        if(_enable_tracking)
+        {
             if(_show_paths & TRACKS_MASK)
             {
                 /* Instead of calling map_render_paths(), we'll draw the new
@@ -815,177 +812,177 @@ track_add(time_t time, gboolean newly_fixed)
                             abs(ty1 - ty2) + (2 * _draw_width));
                 }
             }
+
             MACRO_PATH_INCREMENT_TAIL(_track);
-
             *_track.tail = _pos;
+        }
 
-            /* Calculate distance to route. (point to line) */
-            if(_near_point)
+        /* Calculate distance to route. (point to line) */
+        if(_near_point)
+        {
+            gint route_x1, route_x2, route_y1, route_y2;
+            gint64 route_dist_squared_1 = INT64_MAX;
+            gint64 route_dist_squared_2 = INT64_MAX;
+            gfloat slope;
+
+            route_x1 = _near_point->unitx;
+            route_y1 = _near_point->unity;
+
+            /* Try previous point first. */
+            if(_near_point != _route.head && _near_point[-1].unity)
             {
-                gint route_x1, route_x2, route_y1, route_y2;
-                gint64 route_dist_squared_1 = INT64_MAX;
-                gint64 route_dist_squared_2 = INT64_MAX;
-                gfloat slope;
+                route_x2 = _near_point[-1].unitx;
+                route_y2 = _near_point[-1].unity;
+                slope = (gfloat)(route_y2 - route_y1)
+                    / (gfloat)(route_x2 - route_x1);
 
-                route_x1 = _near_point->unitx;
-                route_y1 = _near_point->unity;
-
-                /* Try previous point first. */
-                if(_near_point != _route.head && _near_point[-1].unity)
+                if(route_x1 == route_x2)
                 {
-                    route_x2 = _near_point[-1].unitx;
-                    route_y2 = _near_point[-1].unity;
-                    slope = (gfloat)(route_y2 - route_y1)
-                        / (gfloat)(route_x2 - route_x1);
-
-                    if(route_x1 == route_x2)
-                    {
-                        /* Vertical line special case. */
-                        route_dist_squared_1 = (_pos.unitx - route_x1)
-                            * (_pos.unitx - route_x1);
-                    }
-                    else
-                    {
-                        route_dist_squared_1 = abs((slope * _pos.unitx)
-                            - _pos.unity + (route_y1 - (slope * route_x1)));
-                        route_dist_squared_1 =
-                            route_dist_squared_1 * route_dist_squared_1
-                            / ((slope * slope) + 1);
-                    }
+                    /* Vertical line special case. */
+                    route_dist_squared_1 = (_pos.unitx - route_x1)
+                        * (_pos.unitx - route_x1);
                 }
-                if(_near_point != _route.tail && _near_point[1].unity)
+                else
                 {
-                    route_x2 = _near_point[1].unitx;
-                    route_y2 = _near_point[1].unity;
-                    slope = (gfloat)(route_y2 - route_y1)
-                        / (gfloat)(route_x2 - route_x1);
-
-                    if(route_x1 == route_x2)
-                    {
-                        /* Vertical line special case. */
-                        route_dist_squared_2 = (_pos.unitx - route_x1)
-                            * (_pos.unitx - route_x1);
-                    }
-                    else
-                    {
-                        route_dist_squared_2 = abs((slope * _pos.unitx)
-                            - _pos.unity + (route_y1 - (slope * route_x1)));
-                        route_dist_squared_2 =
-                            route_dist_squared_2 * route_dist_squared_2
-                            / ((slope * slope) + 1);
-                    }
+                    route_dist_squared_1 = abs((slope * _pos.unitx)
+                        - _pos.unity + (route_y1 - (slope * route_x1)));
+                    route_dist_squared_1 =
+                        route_dist_squared_1 * route_dist_squared_1
+                        / ((slope * slope) + 1);
                 }
+            }
+            if(_near_point != _route.tail && _near_point[1].unity)
+            {
+                route_x2 = _near_point[1].unitx;
+                route_y2 = _near_point[1].unity;
+                slope = (gfloat)(route_y2 - route_y1)
+                    / (gfloat)(route_x2 - route_x1);
 
-                /* Check if our distance from the route is large. */
-                if(MIN(route_dist_squared_1, route_dist_squared_2)
-                        > (2000 * 2000))
+                if(route_x1 == route_x2)
                 {
-                    /* Prevent announcments from occurring. */
-                    announce_thres_unsquared = INT_MAX;
-
-                    if(_autoroute_data.enabled && !_autoroute_data.in_progress)
-                    {
-                        MACRO_BANNER_SHOW_INFO(_window,
-                                _("Recalculating directions..."));
-                        _autoroute_data.in_progress = TRUE;
-                        show_directions = FALSE;
-                        g_idle_add((GSourceFunc)auto_route_dl_idle, NULL);
-                    }
-                    else
-                    {
-                        /* Reset the route to try and find the nearest point.*/
-                        path_reset_route();
-                    }
+                    /* Vertical line special case. */
+                    route_dist_squared_2 = (_pos.unitx - route_x1)
+                        * (_pos.unitx - route_x1);
+                }
+                else
+                {
+                    route_dist_squared_2 = abs((slope * _pos.unitx)
+                        - _pos.unity + (route_y1 - (slope * route_x1)));
+                    route_dist_squared_2 =
+                        route_dist_squared_2 * route_dist_squared_2
+                        / ((slope * slope) + 1);
                 }
             }
 
-            /* Keep the display on. */
-            moving = TRUE;
-        }
-
-        if(_initial_distance_waypoint
-               && (_next_way != _initial_distance_waypoint
-               ||  _next_way_dist_squared > (_initial_distance_from_waypoint
-                                           * _initial_distance_from_waypoint)))
-        {
-            /* We've moved on to the next waypoint, or we're really far from
-             * the current waypoint. */
-            if(_waypoint_banner)
+            /* Check if our distance from the route is large. */
+            if(MIN(route_dist_squared_1, route_dist_squared_2)
+                    > (2000 * 2000))
             {
-                gtk_widget_destroy(_waypoint_banner);
-                _waypoint_banner = NULL;
-            }
-            _initial_distance_from_waypoint = -1.f;
-            _initial_distance_waypoint = NULL;
-        }
+                /* Prevent announcments from occurring. */
+                announce_thres_unsquared = INT_MAX;
 
-        /* Check if we should announce upcoming waypoints. */
-        if(_enable_announce
-                && (_initial_distance_waypoint || _next_way_dist_squared
-                    < (announce_thres_unsquared * announce_thres_unsquared)))
-        {
-            if(show_directions)
-            {
-                if(!_initial_distance_waypoint)
+                if(_autoroute_data.enabled && !_autoroute_data.in_progress)
                 {
-                    /* First time we're close enough to this waypoint. */
-                    if(_enable_voice
-                            /* And that we haven't already announced it. */
-                            && strcmp(_next_way->desc, _last_spoken_phrase))
-                    {
-                        g_free(_last_spoken_phrase);
-                        _last_spoken_phrase = g_strdup(_next_way->desc);
-                        if(!fork())
-                        {
-                            /* We are the fork child.  Synthesize the voice. */
-                            hildon_play_system_sound(
-                                "/usr/share/sounds/ui-information_note.wav");
-                            sleep(1);
-#               define _voice_synth_path "/usr/bin/flite"
-                            printf("%s %s\n", _voice_synth_path,
-                                    _last_spoken_phrase);
-                            execl(_voice_synth_path, _voice_synth_path,
-                                    "-t", _last_spoken_phrase, (char *)NULL);
-                            exit(0);
-                        }
-                    }
-                    _initial_distance_from_waypoint
-                        = sqrtf(_next_way_dist_squared);
-                    _initial_distance_waypoint = _next_way;
-                    if(_next_wpt && _next_wpt->unity != 0)
-                    {
-                        /* Create a banner for us the show progress. */
-                        _waypoint_banner = hildon_banner_show_progress(
-                                _window, NULL, _next_way->desc);
-                    }
-                    else
-                    {
-                        /* This is the last point in a segment, i.e.
-                         * "Arrive at ..." - just announce. */
-                        MACRO_BANNER_SHOW_INFO(_window, _next_way->desc);
-                    }
+                    MACRO_BANNER_SHOW_INFO(_window,
+                            _("Recalculating directions..."));
+                    _autoroute_data.in_progress = TRUE;
+                    show_directions = FALSE;
+                    g_idle_add((GSourceFunc)auto_route_dl_idle, NULL);
                 }
-                else if(_waypoint_banner);
+                else
                 {
-                    /* We're already close to this waypoint. */
-                    gdouble fraction = 1.f - (sqrtf(_next_way_dist_squared)
-                            / _initial_distance_from_waypoint);
-                    BOUND(fraction, 0.f, 1.f);
-                    hildon_banner_set_fraction(
-                            HILDON_BANNER(_waypoint_banner), fraction);
+                    /* Reset the route to try and find the nearest point.*/
+                    path_reset_route();
                 }
             }
-            approaching_waypoint = TRUE;
-        }
-        else if(_next_way_dist_squared > 2 * (_initial_distance_from_waypoint
-                                            * _initial_distance_from_waypoint))
-        {
-            /* We're too far away now - destroy the banner. */
         }
 
-        UNBLANK_SCREEN(moving, approaching_waypoint);
+        /* Keep the display on. */
+        moving = TRUE;
     }
 
+    if(_initial_distance_waypoint
+           && (_next_way != _initial_distance_waypoint
+           ||  _next_way_dist_squared > (_initial_distance_from_waypoint
+                                       * _initial_distance_from_waypoint)))
+    {
+        /* We've moved on to the next waypoint, or we're really far from
+         * the current waypoint. */
+        if(_waypoint_banner)
+        {
+            gtk_widget_destroy(_waypoint_banner);
+            _waypoint_banner = NULL;
+        }
+        _initial_distance_from_waypoint = -1.f;
+        _initial_distance_waypoint = NULL;
+    }
+
+    /* Check if we should announce upcoming waypoints. */
+    if(_enable_announce
+            && (_initial_distance_waypoint || _next_way_dist_squared
+                < (announce_thres_unsquared * announce_thres_unsquared)))
+    {
+        if(show_directions)
+        {
+            if(!_initial_distance_waypoint)
+            {
+                /* First time we're close enough to this waypoint. */
+                if(_enable_voice
+                        /* And that we haven't already announced it. */
+                        && strcmp(_next_way->desc, _last_spoken_phrase))
+                {
+                    g_free(_last_spoken_phrase);
+                    _last_spoken_phrase = g_strdup(_next_way->desc);
+                    if(!fork())
+                    {
+                        /* We are the fork child.  Synthesize the voice. */
+                        hildon_play_system_sound(
+                            "/usr/share/sounds/ui-information_note.wav");
+                        sleep(1);
+#               define _voice_synth_path "/usr/bin/flite"
+                        printf("%s %s\n", _voice_synth_path,
+                                _last_spoken_phrase);
+                        execl(_voice_synth_path, _voice_synth_path,
+                                "-t", _last_spoken_phrase, (char *)NULL);
+                        exit(0);
+                    }
+                }
+                _initial_distance_from_waypoint
+                    = sqrtf(_next_way_dist_squared);
+                _initial_distance_waypoint = _next_way;
+                if(_next_wpt && _next_wpt->unity != 0)
+                {
+                    /* Create a banner for us the show progress. */
+                    _waypoint_banner = hildon_banner_show_progress(
+                            _window, NULL, _next_way->desc);
+                }
+                else
+                {
+                    /* This is the last point in a segment, i.e.
+                     * "Arrive at ..." - just announce. */
+                    MACRO_BANNER_SHOW_INFO(_window, _next_way->desc);
+                }
+            }
+            else if(_waypoint_banner);
+            {
+                /* We're already close to this waypoint. */
+                gdouble fraction = 1.f - (sqrtf(_next_way_dist_squared)
+                        / _initial_distance_from_waypoint);
+                BOUND(fraction, 0.f, 1.f);
+                hildon_banner_set_fraction(
+                        HILDON_BANNER(_waypoint_banner), fraction);
+            }
+        }
+        approaching_waypoint = TRUE;
+    }
+    else if(_next_way_dist_squared > 2 * (_initial_distance_from_waypoint
+                                        * _initial_distance_from_waypoint))
+    {
+        /* We're too far away now - destroy the banner. */
+    }
+
+    UNBLANK_SCREEN(moving, approaching_waypoint);
+
     /* Maybe update the track database. */
     {
         static time_t last_track_db_update = 0;
@@ -1053,10 +1050,9 @@ track_insert_break(gboolean temporary)
                     0, /* start at 0 degrees. */
                     360 * 64);
     }
-    else if(!temporary)
-    {
-        MACRO_BANNER_SHOW_INFO(_window, _("Break already inserted."));
-    }
+
+    /* Update the track database. */
+    path_update_track_in_db();
 
     vprintf("%s(): return\n", __PRETTY_FUNCTION__);
 }
@@ -1119,8 +1115,7 @@ find_nearest_waypoint(gint unitx, gint unity)
 }
 
 static gboolean
-origin_type_selected(GtkWidget *toggle,
-        OriginToggleInfo *oti)
+origin_type_selected(GtkWidget *toggle, RouteDownloadInfo *oti)
 {
     printf("%s()\n", __PRETTY_FUNCTION__);
 
@@ -1133,6 +1128,63 @@ origin_type_selected(GtkWidget *toggle,
     return TRUE;
 }
 
+static gboolean
+route_download_swap(GtkWidget *button, RouteDownloadInfo *oti)
+{
+    gchar *old_origin;
+    printf("%s()\n", __PRETTY_FUNCTION__);
+
+    /* Save the old origin. */
+    old_origin = g_strdup(gtk_entry_get_text(GTK_ENTRY(oti->txt_from)));
+
+    /* Set the origin text field equal to the current destination. */
+    gtk_entry_set_text(GTK_ENTRY(oti->txt_from),
+            gtk_entry_get_text(GTK_ENTRY(oti->txt_to)));
+
+    /* Set the contents of the destination text field. */
+    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(oti->rad_use_gps)))
+    {
+        /* "Use GPS Location" is enabled - set Destination to GPS Location */
+        gchar buffer[80];
+        gchar strlat[32];
+        gchar strlon[32];
+        g_ascii_formatd(strlat, 32, "%.06f", _gps.lat);
+        g_ascii_formatd(strlon, 32, "%.06f", _gps.lon);
+        snprintf(buffer, sizeof(buffer), "%s, %s", strlat, strlon);
+        gtk_entry_set_text(GTK_ENTRY(oti->txt_to), buffer);
+    }
+    else if(gtk_toggle_button_get_active(
+                GTK_TOGGLE_BUTTON(oti->rad_use_route)))
+    {
+        /* "Use End of Route" is enabled - set Destination to start of route */
+        gchar buffer[80];
+        gchar strlat[32];
+        gchar strlon[32];
+        Point *p;
+        gdouble lat, lon;
+
+        /* Use first non-zero route point. */
+        for(p = _route.head; !p->unity; p++) { }
+
+        unit2latlon(p->unitx, p->unity, lat, lon);
+        g_ascii_formatd(strlat, 32, "%.06f", lat);
+        g_ascii_formatd(strlon, 32, "%.06f", lon);
+        snprintf(buffer, sizeof(buffer), "%s, %s", strlat, strlon);
+        gtk_entry_set_text(GTK_ENTRY(oti->txt_to), buffer);
+    }
+    else if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(oti->rad_use_text)))
+    {
+        /* "Origin" is enabled - just use the text. */
+        gtk_entry_set_text(GTK_ENTRY(oti->txt_to), old_origin);
+    }
+
+    g_free(old_origin);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(oti->rad_use_text), TRUE);
+
+    vprintf("%s(): return TRUE\n", __PRETTY_FUNCTION__);
+    return TRUE;
+}
+
 /**
  * Display a dialog box to the user asking them to download a route.  The
  * "From" and "To" textfields may be initialized using the first two
@@ -1150,7 +1202,8 @@ route_download(gchar *to)
     static GtkWidget *table = NULL;
     static GtkWidget *label = NULL;
     static GtkWidget *txt_source_url = NULL;
-    static OriginToggleInfo oti;
+    static GtkWidget *btn_swap = NULL;
+    static RouteDownloadInfo oti;
     printf("%s()\n", __PRETTY_FUNCTION__);
 
     conic_recommend_connected();
@@ -1183,7 +1236,7 @@ route_download(gchar *to)
         gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
         gtk_table_attach(GTK_TABLE(table),
                 txt_source_url = gtk_entry_new(),
-                1, 4, 0, 1, GTK_EXPAND | GTK_FILL, 0, 2, 4);
+                1, 5, 0, 1, GTK_EXPAND | GTK_FILL, 0, 2, 4);
         gtk_entry_set_width_chars(GTK_ENTRY(txt_source_url), 25);
 
         /* Use GPS Location. */
@@ -1206,13 +1259,13 @@ route_download(gchar *to)
         gtk_table_attach(GTK_TABLE(table),
                 oti.chk_auto = gtk_check_button_new_with_label(
                     _("Auto-Update")),
-                3, 4, 1, 2, GTK_FILL, 0, 2, 4);
+                3, 5, 1, 2, GTK_FILL, 0, 2, 4);
 
         /* Avoid Highways. */
         gtk_table_attach(GTK_TABLE(table),
                 oti.chk_avoid_highways = gtk_check_button_new_with_label(
                     _("Avoid Highways")),
-                3, 4, 2, 3, GTK_FILL, 0, 2, 4);
+                3, 5, 2, 3, GTK_FILL, 0, 2, 4);
 
 
         /* Origin. */
@@ -1247,6 +1300,10 @@ route_download(gchar *to)
         g_object_set(G_OBJECT(oti.txt_to), HILDON_AUTOCAP, FALSE, NULL);
 #endif
 
+        /* Swap button. */
+        gtk_table_attach(GTK_TABLE(table),
+                btn_swap = gtk_button_new_with_label("Swap"),
+                4, 5, 3, 5, GTK_FILL, GTK_FILL, 2, 4);
 
         /* Set up auto-completion. */
         from_comp = gtk_entry_completion_new();
@@ -1266,6 +1323,8 @@ route_download(gchar *to)
                           G_CALLBACK(origin_type_selected), &oti);
         g_signal_connect(G_OBJECT(oti.rad_use_text), "toggled",
                           G_CALLBACK(origin_type_selected), &oti);
+        g_signal_connect(G_OBJECT(btn_swap), "clicked",
+                          G_CALLBACK(route_download_swap), &oti);
 
         gtk_widget_set_sensitive(oti.chk_auto, FALSE);
     }
